#include<stdio.h>
#include<AL/alut.h>
#include "audio.h"

int win_vyska, win_sirka;

#define NUM_SOURCES 50

static ALuint cink1, cink2, ding, tutwall, tutsip, buch, ssss, ss, tata, plam;

static ALuint sources[NUM_SOURCES];
static int cursource = 0;

static ALuint sip_source[2];
static char sip_state[2];
static ALfloat sip_pos[2][3];
static ALfloat sip_length[2];
static int sip_x[2];

static ALuint corner_source[4];

static void playbuffer(float x, float y, ALuint buffer, ALfloat gain)
{
  x /= win_sirka;
  y /= win_sirka;

  alSourceStop(sources[cursource]);
  alSource3f(sources[cursource], AL_POSITION, x, y, 0);
  alSourcei(sources[cursource], AL_BUFFER, buffer);
  alSourcef(sources[cursource], AL_GAIN, gain);
  alSourcePlay(sources[cursource]);

  cursource++;
  if(cursource == NUM_SOURCES) cursource = 0;
}

void audio_init(int argc, char **argv)
{
  int i;

  alutInit(&argc, argv);

  alListener3f(AL_POSITION, 0.5, ((ALfloat)win_vyska)/win_sirka/2, 0);

  cink1 = alutCreateBufferFromFile("sounds/cink1.wav");
  cink2 = alutCreateBufferFromFile("sounds/cink2.wav");
  ding = alutCreateBufferFromFile("sounds/ding.wav");
  tutwall = alutCreateBufferWaveform (ALUT_WAVEFORM_SINE,
				      400, 0, 0.1);
  tutsip = alutCreateBufferWaveform (ALUT_WAVEFORM_SINE,
				     440, 0, 0.1);
  buch = alutCreateBufferFromFile("sounds/buch.wav");
  ssss = alutCreateBufferWaveform (ALUT_WAVEFORM_WHITENOISE,
				   1, 0, 1);
  ss = alutCreateBufferFromFile("sounds/ss.wav");
  tata = alutCreateBufferFromFile("sounds/tata.wav");
  plam = alutCreateBufferFromFile("sounds/plam.wav");

  alGenSources(2, sip_source);
  sip_state[0] = sip_state[1] = 0;

  alGenSources(4, corner_source);
  for(i=0; i<4; i++)
    alSourcei(corner_source[i], AL_BUFFER, ding);
  alSource3f(corner_source[0], AL_POSITION, 0, 0, 0);
  alSource3f(corner_source[1], AL_POSITION, 1, 0, 0);
  alSource3f(corner_source[2], AL_POSITION, 0, ((ALfloat)win_vyska)/win_sirka, 0);
  alSource3f(corner_source[3], AL_POSITION, 1, ((ALfloat)win_vyska)/win_sirka, 0);

  alGenSources(NUM_SOURCES, sources);
}

void audio_stop()
{
  int i;

  for(i=0; i<NUM_SOURCES; i++)
    alSourceStop(sources[i]);
  audio_sipstop(0);
  audio_sipstop(1);
}

static void setsipgain(int side, char force)
{
  ALfloat gain;

  if(side == 1) gain = sip_x[side] + sip_length[side];
  else gain = win_sirka - sip_x[side] + sip_length[side];
  if(gain >= sip_length[side]) gain = 1;
  else if(gain <= 0){
    sip_state[side] = 3;
    alSourceStop(sip_source[side]);
    return;
  }
  else gain /= sip_length[side];

  if(gain == 1 && !force) return;

  if(sip_state[side] == 1) gain *= 0.3;

  alSourcef(sip_source[side], AL_GAIN, gain);
}

void audio_sipstart(int side, float x, float y, int length)
{
  if(side == 0) x += length;
  sip_length[side] = length;
  sip_x [side] = x;

  playbuffer(x, y, buch, 0.5);

  sip_pos[side][0] = ((ALfloat)x)/win_sirka;
  sip_pos[side][1] = ((ALfloat)y)/win_sirka;
  sip_pos[side][2] = 0;

  sip_state[side] = 1;

  alSourcefv(sip_source[side],AL_POSITION, sip_pos[side]);
  alSourcei(sip_source[side], AL_BUFFER, ss);
  alSourcei(sip_source[side], AL_LOOPING,AL_TRUE);
  alSourcePlay(sip_source[side]);

  setsipgain(side, 1);
}

void audio_sipmove(int side, float x, char block)
{
  char force = 0;

  if(sip_state[side] == 3) return;

  if(side == 0) x += sip_length[side];

  sip_x [side] = x;
  sip_pos[side][0] = ((ALfloat)x)/win_sirka;
  alSourcefv(sip_source[side],AL_POSITION, sip_pos[side]);

  if(block && sip_state[side] == 1){
    force = 1;
    sip_state[side] = 2;
    alSourceStop(sip_source[side]);
    alSourcei(sip_source[side], AL_BUFFER, ssss);
    alSourcePlay(sip_source[side]);
  }
  else if(!block && sip_state[side] == 2){
    force = 1;
    sip_state[side] = 1;
    alSourceStop(sip_source[side]);
    alSourcei(sip_source[side], AL_BUFFER, ss);
    alSourcePlay(sip_source[side]);
  }

  setsipgain(side, force);
}

void audio_sipstop(int side)
{
  alSourceStop(sip_source[side]);
  sip_state[side] = 0;
}

void audio_deskbounce(int corner, float speed)
{
  ALfloat gain;

  if(speed < 0) speed *= -1;
  gain = speed-3;
  if(gain < 0) gain = 0;
  gain /= 40;
  if(gain > 1) gain = 1;

  alSourceStop(corner_source[corner]);
  if(!gain) return;
  alSourcef(corner_source[corner], AL_GAIN, gain);
  alSourcePlay(corner_source[corner]);
}

void audio_start()
{
  playbuffer(win_vyska/2, win_sirka/2, plam, 0.5);
}
void audio_sidebounce(char side, float x, float y)
{
  if(side == 0) playbuffer(x, y, cink1, 1);
  else playbuffer(x, y, cink2, 1);
}
void audio_wallbounce(float x, float y)
{
  playbuffer(x, y, tutwall, 1);
}
void audio_sipbounce(float x, float y)
{
  playbuffer(x, y, tutsip, 1);
}
void audio_sipreload(float x, float y)
{
  playbuffer(x, y, tata, 1);
}
